@@ -1,6 +1,7 @@
 use std::any::Any;
 use std::cell::{Cell, UnsafeCell};
 use std::collections::{HashMap, HashSet, VecDeque};
+use std::marker::PhantomData;
 use std::panic::{self, AssertUnwindSafe};
 use std::sync::atomic::{AtomicU64, Ordering};
 use std::sync::{Arc, Condvar, Mutex, MutexGuard, Weak};
@@ -13,6 +14,8 @@ use crate::coroutine::{self, Coroutine};
 use crate::error::{JoinError, PanicError};
 use crate::runtime::Scheduler;
 
+pub mod mpsc;
+
 static TID_COUNTER: AtomicU64 = AtomicU64::new(1);
 
 thread_local! {
@@ -131,6 +134,8 @@ pub(crate) struct Task {
 
     aborting: bool,
 
+    yielding: bool,
+
     // Newly spawned coroutines.
     spawned_coroutines: Vec<ptr::NonNull<Coroutine>>,
 
@@ -179,6 +184,7 @@ impl Task {
             main: co,
             running: Cell::new(true),
             aborting: false,
+            yielding: false,
             spawned_coroutines: vec![co],
             running_coroutines: VecDeque::with_capacity(5),
             yielding_coroutines: Vec::with_capacity(5),
@@ -273,7 +279,7 @@ impl Task {
         self.running_coroutines.extend(self.spawned_coroutines.drain(..));
         self.running_coroutines.extend(self.yielding_coroutines.drain(..));
         self.unblock(false);
-        while !self.spawned_coroutines.is_empty() || !self.running_coroutines.is_empty() {
+        while !(self.yielding || (self.spawned_coroutines.is_empty() && self.running_coroutines.is_empty())) {
             while let Some(co) = self.spawned_coroutines.pop() {
                 self.run_coroutine(co);
             }
@@ -281,6 +287,7 @@ impl Task {
                 self.run_coroutine(co);
             }
         }
+        self.yielding = false;
         if !self.yielding_coroutines.is_empty() {
             SchedFlow::Yield
         } else if self.blocking_coroutines.is_empty() {
@@ -315,6 +322,18 @@ impl Task {
         co.suspend();
     }
 
+    pub fn yield_coroutine(&mut self, mut co: ptr::NonNull<Coroutine>) {
+        self.yielding_coroutines.push(co);
+        let co = unsafe { co.as_mut() };
+        co.suspend();
+    }
+
+    fn yield_task(&mut self) {
+        self.yielding = true;
+        let co = coroutine::current();
+        self.yield_coroutine(co);
+    }
+
     fn wake(&self, co: ptr::NonNull<Coroutine>, panicking: Option<&'static str>) -> bool {
         let mut unblockings = self.unblocking_coroutines.lock().unwrap();
         let waking = if unblockings.is_empty() && !self.running.get() {
@@ -345,9 +364,19 @@ enum SessionState<T: Send + 'static> {
 
 struct SessionJoint<T: Send + 'static> {
     state: Mutex<SessionState<T>>,
+
+    // - Use Option to avoid create os resource if not needed.
+    // - Use `UnsafeCell` to construct one if necessary.
+    // - Store it outside state so we can access it after unlocking.
     condvar: UnsafeCell<Option<Condvar>>,
 }
 
+// SAFETY: There are two immutable accessors.
+unsafe impl<T: Send> Sync for SessionJoint<T> {}
+
+// SAFETY: Normally, two immutable accessors are distributed to different tasks or threads.
+unsafe impl<T: Send> Send for SessionJoint<T> {}
+
 impl<T: Send + 'static> SessionJoint<T> {
     fn new() -> Arc<Self> {
         Arc::new(SessionJoint { state: Mutex::new(SessionState::Empty), condvar: UnsafeCell::new(None) })
@@ -458,22 +487,36 @@ impl<T: Send + 'static> SessionJoint<T> {
 /// Session provides method to block current coroutine until waking by [SessionWaker].
 pub struct Session<T: Send + 'static> {
     joint: Arc<SessionJoint<T>>,
+    marker: PhantomData<NotSendable>,
 }
 
 /// SessionWaker provides method to wake associated [Session].
 pub struct SessionWaker<T: Send + 'static> {
     joint: Arc<SessionJoint<T>>,
     waked: bool,
+    marker: PhantomData<Sendable>,
 }
 
-unsafe impl<T: Send> Send for SessionWaker<T> {}
+struct NotSendable(std::rc::Rc<()>);
+assert_not_impl_any!(NotSendable: Send, Sync);
 
-assert_impl_all!(SessionWaker<()>: Send);
-assert_not_impl_any!(Session<()>: Send);
+struct Sendable(std::rc::Rc<()>);
+unsafe impl Send for Sendable {}
+assert_impl_all!(Sendable: Send);
+assert_not_impl_any!(Sendable: Sync);
+
+// SessionWaker should be able to send across tasks and threads.
+assert_impl_all!(SessionWaker<Sendable>: Send);
+
+// SessionWaker should owned by only one task or thread.
+assert_not_impl_any!(SessionWaker<Sendable>: Sync);
+
+// Session should be used only be producing task or thread.
+assert_not_impl_any!(Session<Sendable>: Send, Sync);
 
 impl<T: Send + 'static> Session<T> {
     fn new(joint: Arc<SessionJoint<T>>) -> Session<T> {
-        Session { joint }
+        Session { joint, marker: PhantomData }
     }
 
     /// Waits peer to wake it.
@@ -498,7 +541,7 @@ impl<T: Send> Drop for SessionWaker<T> {
 
 impl<T: Send> SessionWaker<T> {
     fn new(joint: Arc<SessionJoint<T>>) -> SessionWaker<T> {
-        SessionWaker { joint, waked: false }
+        SessionWaker { joint, waked: false, marker: PhantomData }
     }
 
     /// Wakes peer.
@@ -528,6 +571,12 @@ where
     (session, session_waker)
 }
 
+/// Yields task for next scheduling cycle.
+pub fn yield_now() {
+    let t = unsafe { current().as_mut() };
+    t.yield_task();
+}
+
 /// Spawns a concurrent task and returns a [JoinHandle] for it.
 ///
 /// The given fn serves as `main` in spawned task. All other coroutines in that task will be
@@ -540,3 +589,39 @@ where
 {
     Builder::new().spawn(f)
 }
+
+#[cfg(test)]
+mod tests {
+    use std::sync::{Arc, Mutex};
+
+    use pretty_assertions::assert_eq;
+
+    use crate::runtime::Builder;
+    use crate::{coroutine, task};
+
+    #[test]
+    fn yield_now() {
+        let runtime = Builder::default().parallelism(1).build();
+        let five = runtime.spawn(|| {
+            let shared_value = Arc::new(Mutex::new(0));
+            let shared_task_value = shared_value.clone();
+            let shared_coroutine_value = shared_value.clone();
+            coroutine::spawn(move || {
+                let mut value = shared_coroutine_value.lock().unwrap();
+                if *value == 0 {
+                    *value = 6;
+                }
+            });
+            task::spawn(move || {
+                let mut value = shared_task_value.lock().unwrap();
+                if *value == 0 {
+                    *value = 5;
+                }
+            });
+            task::yield_now();
+            let value = shared_value.lock().unwrap();
+            *value
+        });
+        assert_eq!(5, five.join().unwrap());
+    }
+}
